﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Net;
using System.Web;
using System.Web.Mvc;
using Subhastes.Models;

namespace Subhastes.Controllers
{
    public class ProductesController : Controller
    {
        private SubhastesEntities1 db = new SubhastesEntities1();

        // GET: Productes
        public ActionResult Index()
        {
            var productes = db.Productes.Include(p => p.Aeroport);
            return View(productes.ToList());
        }

        // GET: Productes/Details/5
        public ActionResult Details(int? id)
        {
            if (id == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }
            Producte producte = db.Productes.Find(id);
            if (producte == null)
            {
                return HttpNotFound();
            }
            return View(producte);
        }

        // GET: Productes/Create
        public ActionResult Create()
        {
            ViewBag.IdAeroport = new SelectList(db.Aeroports, "IdAeroport", "Nom");
            ViewBag.IdUsuari = new SelectList(db.Usuaris, "IdUsuari", "Nom");
            return View();
        }

        // POST: Productes/Create
        // To protect from overposting attacks, please enable the specific properties you want to bind to, for 
        // more details see http://go.microsoft.com/fwlink/?LinkId=317598.
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create([Bind(Include = "IdProducte,IdAeroport,IdUsuari,Nom,Descripcio,Imatge")] Producte producte, HttpPostedFileBase image)
        {
            if (ModelState.IsValid)
            {
                if (image != null)
                {
                    producte.ImatgeMimeType = image.ContentType;
                    producte.Imatge = new byte[image.ContentLength];
                    image.InputStream.Read(producte.Imatge, 0, image.ContentLength);

                    db.Productes.Add(producte);
                    db.SaveChanges();
                    return RedirectToAction("Index");
                }
            }

            ViewBag.IdAeroport = new SelectList(db.Aeroports, "IdAeroport", "Nom", producte.IdAeroport);
            return View(producte);
        }

        // GET: Productes/Edit/5
        public ActionResult Edit(int? id)
        {
            if (id == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }
            Producte producte = db.Productes.Find(id);
            if (producte == null)
            {
                return HttpNotFound();
            }
            ViewBag.IdAeroport = new SelectList(db.Aeroports, "IdAeroport", "Nom", producte.IdAeroport);
            return View(producte);
        }

        // POST: Productes/Edit/5
        // To protect from overposting attacks, please enable the specific properties you want to bind to, for 
        // more details see http://go.microsoft.com/fwlink/?LinkId=317598.
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit([Bind(Include = "IdProducte,IdAeroport,IdUsuari,Nom,Descripcio,Imatge, ImatgeMimeType")] Producte producte, HttpPostedFileBase newImage)
        {
            if (ModelState.IsValid)
            {
                if (newImage != null)
                {
                    producte.ImatgeMimeType = newImage.ContentType;
                    producte.Imatge = new byte[newImage.ContentLength];
                    newImage.InputStream.Read(producte.Imatge, 0, newImage.ContentLength);
                }

                db.Entry(producte).State = EntityState.Modified;
                db.SaveChanges();
                return RedirectToAction("Index");
            }
            ViewBag.IdAeroport = new SelectList(db.Aeroports, "IdAeroport", "Nom", producte.IdAeroport);
            
            return View(producte);
        }

        // GET: Productes/Delete/5
        public ActionResult Delete(int? id)
        {
            if (id == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }
            Producte producte = db.Productes.Find(id);
            if (producte == null)
            {
                return HttpNotFound();
            }
            return View(producte);
        }

        // POST: Productes/Delete/5
        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public ActionResult DeleteConfirmed(int id)
        {
            Producte producte = db.Productes.Find(id);
            db.Productes.Remove(producte);
            db.SaveChanges();
            return RedirectToAction("Index");
        }


        /// <summary>
        /// 
        /// </summary>
        /// <param name="productId"></param>
        /// <returns></returns>
        public FileContentResult GetImage(int idProducte)
        {
            Producte producte = db.Productes.FirstOrDefault(p => p.IdProducte == idProducte);
            if (producte != null && producte.Imatge!=null && !String.IsNullOrEmpty(producte.ImatgeMimeType))
            {
                return File(producte.Imatge, producte.ImatgeMimeType);
            }
            else
            {
                return null;
            }
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                db.Dispose();
            }
            base.Dispose(disposing);
        }

        public ActionResult Buscar(string pagOrigen, int? objectId)
        {
            List<SelectListItem> items = new List<SelectListItem>();
            foreach (Aeroport aeroport in db.Aeroports)
            {
                items.Add(new SelectListItem { Text = aeroport.Nom, Value = aeroport.IdAeroport.ToString()});
            }

            ViewBag.IdAeroport = items;
            ViewBag.PagOrigen = pagOrigen;
            ViewBag.ObjectId = objectId.HasValue?objectId.Value:0;

            return View("BuscarProducte");
        }

        [HttpPost]
        public ActionResult Buscar([Bind(Include = "IdAeroport")] Aeroport aeroport, int? idProducte, string pagOrigen, int? objectId)
        {
            Aeroport selectedAeroport = null;
            List<SelectListItem> itemsProducte = null;
            if (aeroport.IdAeroport > 0)
            {
                selectedAeroport = db.Aeroports.FirstOrDefault(a => a.IdAeroport == aeroport.IdAeroport);
                itemsProducte = new List<SelectListItem>();
                foreach (Producte item in selectedAeroport.Productes)
                {
                    SelectListItem itemList = new SelectListItem { Text = item.Nom, Value = item.IdProducte.ToString() };
                    itemsProducte.Add(itemList);
                }
            }

            ViewBag.Aeroport = selectedAeroport;
            ViewBag.IdProducte = itemsProducte;
            ViewBag.PagOrigen = pagOrigen;
            ViewBag.ObjectId = objectId.HasValue ? objectId.Value : 0;

            if (idProducte.HasValue) {
                Producte producte = selectedAeroport.Productes.FirstOrDefault(p => p.IdProducte == idProducte);
                if (ViewBag.ObjectId==0)
                    return RedirectToAction(pagOrigen, "Subhastes", new {idProducte = producte.IdProducte, Nom = producte.Nom });
                else
                    return RedirectToAction(pagOrigen, "Subhastes", new {id = ViewBag.ObjectId, idProducte = producte.IdProducte, Nom = producte.Nom });
            }
            else
                return View("BuscarProducte", null);
                 
        }

       
    }
}
