﻿using SubhastesDesktopApp.ServeiSubhastes;
using SubhastesService.Entities;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Data;

namespace SubhastesDesktopApp
{
    /// <summary>
    /// Dona accés a referències d'objectes relacionats amb les dades de l'aplicació com el servei WCF, 
    /// l'usuari actiu i el controller principal. 
    /// 
    /// Fa ús del patró Singleton.
    /// </summary>
    public class ApplicationData:NotifyPropertyClass
    {
        
        private static ApplicationData instance;
       
        private Usuari usuari;
        private ServeiSubhastesClient servei;

        /// <summary>
        /// Retorna sempre la mateixa instancia del objecte. (Singleton).
        /// </summary>
        public static ApplicationData Instance
        {
            get
            {
                if (instance == null)
                    instance = new ApplicationData();

                return instance;
            }
        }
      
        /// <summary>
        /// Referencia al servei WCF.
        /// </summary>
        public ServeiSubhastesClient Servei
        {
            get 
            {
                if (servei == null)
                    servei = new ServeiSubhastesClient();

                return servei; 
            }
        }

        /// <summary>
        /// Referencia a l'usuari actiu.
        /// </summary>
        public Usuari Usuari 
        {
            get { return usuari; }
            set { 
                    usuari = value;
                    SendPropertyChanged("Usuari");
                
                }
        }

        /// <summary>
        /// Referencia al controlador de la vista principal.
        /// </summary>
        public MainWindowController MainController { get; set; }
        



        
    }
}
