﻿using SubhastesService;
using SubhastesService.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Threading;

namespace SubhastesDesktopApp
{
    public class LlistaSubhastesController:NotifyPropertyClass
    {
        #region Constants
        /// <summary>
        /// Determina l'interval del timer de refresc de la llista de subhastes. En milisegons.
        /// </summary>
        public const int INTERVAL_TIMER = 5000;
        #endregion


        #region Declaracions
        /// <summary>
        /// Timer consulta llista de subhastes.
        /// </summary>
        DispatcherTimer timer;

        /// <summary>
        /// Llista amb les subhastes en estat esperant o iniciades.
        /// </summary>
        private List<Subhasta> subhastes;
        #endregion


        #region Constructor
        /// <summary>
        /// Constructor. Cada x segons s'actualitza la llista de subhastes.
        /// </summary>
        public LlistaSubhastesController()
        {
            timer = new DispatcherTimer();
            timer.Interval = new TimeSpan(0, 0, 0, 0, INTERVAL_TIMER);
            timer.Tick += t_Tick;
            
        }
        #endregion


        #region Propietats
        /// <summary>
        /// Retorna les subhastes en estat esperant o iniciades.
        /// </summary>
        public List<Subhasta> Subhastes
        {
            get { return this.subhastes; }
            set
            {
                this.subhastes = value;
                SendPropertyChanged("Subhastes");
            }
        }
        #endregion


        #region Timer_tick
        /// <summary>
        /// Obté la llista de subhastes actives cada x segons.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void t_Tick(object sender, EventArgs e)
        {
            this.GetSubhastes();
        }
        #endregion


        #region Métodes
        /// <summary>
        /// Inicia el timer de refresc de la llista de subhastes.
        /// </summary>
        public void IniciarRefrescSubhastes()
        {
            timer.Start();
        }

        /// <summary>
        /// Atura el timer de refresc de la llista de subhastes.
        /// </summary>
        public void AturarRefrescSubhastes()
        {
            timer.Stop();
        }


        /// <summary>
        /// Consulta al servei y retorna la llista de subhastes actives.
        /// </summary>
        public void GetSubhastes()
        {
            this.Subhastes = ApplicationData.Instance.Servei.GetSubhastes().ToList();
        }


        /// <summary>
        /// Envia la petició de participació en la subhasta indicada.
        /// Retorna un objecte ResultPeticioParticipar.
        /// </summary>
        /// <param name="subhasta"></param>
        /// <returns></returns>
        public ResultPeticioParticipar Participar(Subhasta subhasta)
        {
            ResultPeticioParticipar result = ApplicationData.Instance.Servei.Participar(ApplicationData.Instance.Usuari, subhasta.IdSubhasta);
            if (result.CodiError == 0)
            {
                ApplicationData.Instance.MainController.Subhasta.CurrentSubhasta = result.Subhasta;
                ApplicationData.Instance.MainController.IsVistaLlistaSubhastesVisible = false;
                ApplicationData.Instance.MainController.IsVistaSubhastaVisible = true;
            }
            return result;
        }
        #endregion




    }
}
