﻿using SubhastesService.Entities;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SubhastesDesktopApp
{
    public class MainWindowController:NotifyPropertyClass
    {
        #region Declaracions
        private bool isVistaNouUsuariVisible;
        private bool isVistaAutenticacioVisible;
        private bool isVistaLlistaSubhastesVisible;
        private bool isVistaSubhastaVisible;
        #endregion


        #region Constructor
        public MainWindowController()
        {
            this.Autenticacio = new AutenticacioController();
            this.LlistaSubhastes = new LlistaSubhastesController();
            this.RegistreUsuari = new RegistreUsuariController();
            this.Subhasta = new SubhastaController();

            this.IsVistaAutenticacioVisible = true;
            
        }

        #endregion


        #region Propietats
        /// <summary>
        /// Determina la vista que es visible actualment.
        /// </summary>
        public EnumCurrentVista CurrentVista { get; set; }

        /// <summary>
        /// Referencia al controller de la vista d'autenticació.
        /// </summary>
        public AutenticacioController Autenticacio { get; set; }

        /// <summary>
        /// Referencia al controller de la llista de subhastes.
        /// </summary>
        public LlistaSubhastesController LlistaSubhastes { get; set; }

        /// <summary>
        /// Referencia al controller de la vista de registre d'un nou usuari.
        /// </summary>
        public RegistreUsuariController RegistreUsuari { get; set; }

        /// <summary>
        /// Referencia al controlador de la vista de la subhasta.
        /// </summary>
        public SubhastaController Subhasta { get; set; }


        /// <summary>
        /// Determina si es visible la vista d'autenticació d'usuari.
        /// </summary>
        public bool IsVistaAutenticacioVisible
        {
            get { return this.isVistaAutenticacioVisible; }
            set { 
                    this.isVistaAutenticacioVisible = value;
                    if (value)
                        this.CurrentVista = EnumCurrentVista.Autenticacio;
                    SendPropertyChanged("IsVistaAutenticacioVisible");
                }
        }

        /// <summary>
        /// Determina si es visible la vista que llista les subhastes.
        /// </summary>
        public bool IsVistaLlistaSubhastesVisible
        {
            get { return this.isVistaLlistaSubhastesVisible; }
            set { 
                    this.isVistaLlistaSubhastesVisible = value;
                    if (value)
                        this.CurrentVista = EnumCurrentVista.LlistaSubhastes;
                    SendPropertyChanged("IsVistaLlistaSubhastesVisible");
                }
        }

        /// <summary>
        /// Determina si es visible la vista per registrar un nou usuari.
        /// </summary>
        public bool IsVistaNouUsuariVisible
        {
            get { return this.isVistaNouUsuariVisible; }
            set { 
                    this.isVistaNouUsuariVisible = value;
                    if (value)
                        this.CurrentVista = EnumCurrentVista.NouUsuari;
                    SendPropertyChanged("IsVistaNouUsuariVisible");
                }
        }

        /// <summary>
        /// Determina si es visible la vista de la subhasta.
        /// </summary>
        public bool IsVistaSubhastaVisible
        {
            get { return isVistaSubhastaVisible; }
            set { 
                    isVistaSubhastaVisible = value;
                    if (value)
                        this.CurrentVista = EnumCurrentVista.Subhasta;
                    SendPropertyChanged("IsVistaSubhastaVisible");
                }
        }

        #endregion


        /// <summary>
        /// Fa visible la vista anterior a la vista actual.
        /// </summary>
        public void GoToVistaAnterior()
        {
            switch (this.CurrentVista)
            {
                
                case EnumCurrentVista.NouUsuari:
                    this.IsVistaNouUsuariVisible = false;
                    this.IsVistaAutenticacioVisible = true;
                    break;
                case EnumCurrentVista.LlistaSubhastes:
                    this.IsVistaLlistaSubhastesVisible = false;
                    this.IsVistaAutenticacioVisible = true;
                    break;
                case EnumCurrentVista.Subhasta:
                    this.IsVistaSubhastaVisible = false;
                    this.IsVistaLlistaSubhastesVisible = true;
                    break;
             
            }
        }
        
       
    }
}
