﻿using SubhastesService;
using SubhastesService.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SubhastesDesktopApp
{
    public class RegistreUsuariController:NotifyPropertyClass
    {
        #region Declaracions
        private string nom;
        private string correu;
        private string direccio;
        private string poblacio;
        private string codiPostal;
        private string password;
        #endregion


        #region Propietats
        public string Nom
        {
            get { return nom; }
            set { 
                    nom = value;
                    SendPropertyChanged("Nom");
                }
        }
        
        public string Correu
        {
            get { return correu; }
            set { 
                    correu = value;
                    SendPropertyChanged("Correu");
                }
        }

        public string Direccio
        {
            get { return direccio; }
            set { 
                    direccio = value;
                    SendPropertyChanged("Direccio");
                }
        }

        public string Poblacio
        {
            get { return poblacio; }
            set { 
                    poblacio = value;
                    SendPropertyChanged("Poblacio");
                }
        }

        public string CodiPostal
        {
            get { return codiPostal; }
            set { 
                    codiPostal = value;
                    SendPropertyChanged("CodiPostal");
                }
        }

        public string Password
        {
            get { return password; }
            set { 
                    password = value;
                    SendPropertyChanged("Password");
                }
        }
        #endregion

        /// <summary>
        /// Posa en blanc totes les propietats.
        /// </summary>
        public void Clear()
        {
            Nom = "";
            Correu = "";
            Direccio = "";
            Poblacio = "";
            CodiPostal = "";
            Password = "";

        }


        /// <summary>
        /// Dona d'alta l'usuari per poder accedir a l'aplicació d'escriptori.
        /// </summary>
        /// <returns></returns>
        public ResultPeticio RegistrarUsuari()
        {
            ResultPeticioAutenticar result = new ResultPeticioAutenticar();

            if (String.IsNullOrEmpty(Nom) || 
                String.IsNullOrEmpty(Correu) || 
                String.IsNullOrEmpty("Password"))
            {
                result.CodiError = -1;
                result.MissatgeError ="El parámetres Nom, Correu i Password son obligatoris";
                return result;
            }

            if (Correu.IndexOf("@") < 0 || Correu.IndexOf(".") < 0)
            {
                result.CodiError = -2;
                result.MissatgeError = "El camp correu es incorrecte";
                return result;
             
            }
            else
            {
                int posDotToken = Correu.IndexOf(".");
                
                if (Correu.Substring(posDotToken + 1).Length < 2)
                {
                    result.CodiError = -2;
                    result.MissatgeError = "El camp correu es incorrecte";
                    return result;
                }
            }

            if (Password.Length < 6)
            {
                result.CodiError = -3;
                result.MissatgeError = "El password ha de tenir una longitud mínima de 6 caracters.";
                return result;
            }


            Usuari usuari = new Usuari
            {
                Nom = nom,
                Correu = correu,
                Direccio = direccio,
                Poblacio = poblacio,
                CP = codiPostal,
                Password = password
            };


            result = ApplicationData.Instance.Servei.RegistrarUsuari(usuari);
            if (result.CodiError==0)
            {
                ApplicationData.Instance.Usuari = result.Usuari;
                ApplicationData.Instance.MainController.IsVistaNouUsuariVisible = false;
                ApplicationData.Instance.MainController.IsVistaLlistaSubhastesVisible = true;
                    
            }

            return result;
        }
        
        
        
        
    }
}
