﻿using SubhastesService.Entities;
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SubhastesService
{
    public class DlAutenticacio: DataLayer
    {

        public ResultPeticioAutenticar Autenticar(string correu, string password)
        {
            ResultPeticioAutenticar result = new ResultPeticioAutenticar();

            using (SqlConnection conn = this.GetConnection())
            {
                conn.Open();
                
                StringBuilder sb = new StringBuilder();
                sb.AppendLine("SELECT IdUsuari, Nom, Password, Correu ");
                sb.AppendLine("FROM Usuari ");
                sb.AppendLine("WHERE Correu = @Correu AND Password = @Password ");

                SqlCommand command = conn.CreateCommand();
                command.Parameters.AddWithValue("Correu", correu);
                command.Parameters.AddWithValue("Password", password);
                command.CommandText = sb.ToString();
                IDataReader reader = command.ExecuteReader();

                if (reader.Read())
                {
                    result.Usuari = new Usuari();
                    result.Usuari.IdUsuari = reader.GetInt32(0);
                    result.Usuari.Nom = reader.GetString(1);
                    result.Usuari.Password = reader.GetString(2);
                    result.Usuari.Correu = reader.GetString(3);
                }
                else
                {
                    result.CodiError = -2;
                    result.MissatgeError = "Usuari " + correu + " no autenticat";
                }
                conn.Close();
            }
            
            return result;
        }


        public ResultPeticioAutenticar RegistrarUsuari(Usuari usuari)
        {
            ResultPeticioAutenticar result = new ResultPeticioAutenticar();
            using (SqlConnection conn = this.GetConnection()) 
            {
                conn.Open();

                if (IsExisteixUsuari(usuari))
                {
                    result.CodiError = -1;
                    result.MissatgeError = "El usuari " + usuari.Correu + " ja existeix";
                    return result;
                }

                StringBuilder sb = new StringBuilder();
                sb.AppendLine("INSERT INTO Usuari (Nom, Direccio, Poblacio, CP, Password, Correu ) ");
                sb.AppendLine("VALUES (@Nom, @Direccio, @Poblacio, @CP, @Password, @Correu) ");

                SqlCommand command = conn.CreateCommand();
                command.Parameters.AddWithValue("Nom", usuari.Nom);
                command.Parameters.AddWithValue("Direccio", String.IsNullOrEmpty(usuari.Direccio) ? "" : usuari.Direccio);
                command.Parameters.AddWithValue("Poblacio", String.IsNullOrEmpty(usuari.Poblacio) ? "" : usuari.Poblacio);
                command.Parameters.AddWithValue("CP", String.IsNullOrEmpty(usuari.CP) ? "" : usuari.CP);
                command.Parameters.AddWithValue("Password", usuari.Password);
                command.Parameters.AddWithValue("Correu", usuari.Correu);
                command.CommandText = sb.ToString();
                command.ExecuteNonQuery();

                //S'obté el id assignat al nou registre.
                command.CommandText = "SELECT MAX(IdUsuari) FROM Usuari ";
                Object idUsuari = command.ExecuteScalar();
                usuari.IdUsuari = idUsuari != null ? Convert.ToInt32(idUsuari) : 0;
                
                conn.Close();
                result.Usuari = usuari;
                
            }

            return result;
        }

        /// <summary>
        /// Determina ja existeix l'usuari indicat en la base de dades en funció de la propietat correu.
        /// </summary>
        /// <param name="usuari"></param>
        /// <returns></returns>
        private bool IsExisteixUsuari(Usuari usuari)
        {
            using (SqlConnection conn = this.GetConnection())
            {
                conn.Open();
                
                StringBuilder sb = new StringBuilder();
                sb.AppendLine("SELECT IdUsuari ");
                sb.AppendLine("FROM Usuari ");
                sb.AppendLine("WHERE Correu=@Correu ");
                SqlCommand command = conn.CreateCommand();
                command.Parameters.AddWithValue("Correu", usuari.Correu);
                command.CommandText = sb.ToString();
                IDataReader reader = command.ExecuteReader();

                return reader.Read();
            }
        }
    }


    
}
