﻿using SubhastesService.Entities;
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SubhastesService
{
    public class DlSubhasta:DataLayer
    {
        /// <summary>
        /// Retorna la capçalera (Sense els participants) de les subhastes no finalitzades. 
        /// </summary>
        /// <returns></returns>
        public List<Subhasta> GetSubhastes()
        {
            List<Subhasta> result = new List<Subhasta>();

            using (SqlConnection conn = this.GetConnection()) 
            {
                conn.Open();

                StringBuilder sb = new StringBuilder();
                sb.AppendLine("SELECT IdSubhasta, DataInici, DataFi, PreuSortida, PreuActual, MaxParticipants, Estat, ");
	            sb.AppendLine("p.IdProducte, p.Nom, p.Descripcio, p.Imatge, p.ImatgeMimeType, ");
	            sb.AppendLine("a.IdAeroport, a.Nom ");
                sb.AppendLine("FROM Subhasta s ");
                sb.AppendLine("INNER JOIN Producte p ON s.IdProducte = p.IdProducte ");
                sb.AppendLine("INNER JOIN Aeroport a ON p.IdAeroport = a.IdAeroport ");
                sb.AppendLine("WHERE Estat !=@Estat");

                SqlCommand command = conn.CreateCommand();
                command.CommandText = sb.ToString();
                command.Parameters.AddWithValue("Estat", (short)enumEstatSubhasta.Finalitzada);
                command.CommandText = sb.ToString();
                IDataReader reader = command.ExecuteReader();
                while (reader.Read())
                {
                    Subhasta subhasta = new Subhasta();
                    subhasta.IdSubhasta = reader.GetInt32(0);
                    subhasta.DataInici = reader.GetDateTime(1);
                    subhasta.DataFi = reader.GetDateTime(2);
                    subhasta.PreuSortida = reader.GetDecimal(3);
                    subhasta.PreuActual = reader.IsDBNull(4) ? 0 : reader.GetDecimal(4);
                    subhasta.MaxParticipants = reader.GetInt32(5);
                    subhasta.Estat = reader.GetInt16(6);

                    Producte producte = new Producte();
                    producte.IdProducte = reader.GetInt32(7);
                    producte.Nom = reader.GetString(8);
                    producte.Descripcio = reader.GetString(9);

                    if (!reader.IsDBNull(10))
                    {
                        byte[] bytes = (byte[])reader.GetValue(10);
                        producte.Imatge = bytes;
                        producte.ImatgeMimeType = reader.GetString(11);
                    }

                    subhasta.Producte = producte;

                    Aeroport aeroport = new Aeroport();
                    aeroport.IdAeroport = reader.GetInt32(12);
                    aeroport.Nom = reader.GetString(13);

                    producte.Aeroport = aeroport;

                    result.Add(subhasta);
                }

            }

            return result;
        }


        /// <summary>
        /// Retorna la subhasta indicada. (Sense la informació de les licitacions).
        /// </summary>
        /// <param name="idSubhasta"></param>
        /// <returns></returns>
        public Subhasta GetSubhasta(int idSubhasta)
        {
            Subhasta result = null;

            using (SqlConnection conn = this.GetConnection())
            {
                conn.Open();

                StringBuilder sb = new StringBuilder();
                sb.AppendLine("SELECT IdSubhasta, DataInici, DataFi, PreuSortida, PreuActual, MaxParticipants, Estat, ");
                sb.AppendLine("p.IdProducte, p.Nom, p.Descripcio, p.Imatge, p.ImatgeMimeType, ");
                sb.AppendLine("a.IdAeroport, a.Nom ");
                sb.AppendLine("FROM Subhasta s ");
                sb.AppendLine("INNER JOIN Producte p ON s.IdProducte = p.IdProducte ");
                sb.AppendLine("INNER JOIN Aeroport a ON p.IdAeroport = a.IdAeroport ");
                sb.AppendLine("WHERE IdSubhasta=@IdSubhasta ");

                SqlCommand command = conn.CreateCommand();
                command.CommandText = sb.ToString();
                command.Parameters.AddWithValue("IdSubhasta", idSubhasta);
                command.CommandText = sb.ToString();
                IDataReader reader = command.ExecuteReader();
                if (reader.Read())
                {
                    result = new Subhasta();
                    result.IdSubhasta = reader.GetInt32(0);
                    result.DataInici = reader.GetDateTime(1);
                    result.DataFi = reader.GetDateTime(2);
                    result.PreuSortida = reader.GetDecimal(3);
                    result.PreuActual = reader.GetDecimal(4);
                    result.MaxParticipants = reader.GetInt32(5);
                    result.Estat = reader.GetInt16(6);

                    Producte producte = new Producte();
                    producte.IdProducte = reader.GetInt32(7);
                    producte.Nom = reader.GetString(8);
                    producte.Descripcio = reader.GetString(9);

                    if (!reader.IsDBNull(10)) { 
                        byte[] bytes = (byte[])reader.GetValue(10);
                        producte.Imatge = bytes;
                        producte.ImatgeMimeType = reader.GetString(11);
                    }
                    result.Producte = producte;

                    Aeroport aeroport = new Aeroport();
                    aeroport.IdAeroport = reader.GetInt32(12);
                    aeroport.Nom = reader.GetString(13);

                    producte.Aeroport = aeroport;
                }

            }

            return result;
        }




        /// <summary>
        /// Incorpora l'usuari indicat a una subhasta, retorna un objecte ResultPeticioParticipar 
        /// amb el resultat de la petició d'incorporació.
        /// </summary>
        /// <param name="usuari"></param>
        /// <param name="idSubhasta"></param>
        /// <returns></returns>
        public ResultPeticioParticipar Participar(Usuari usuari, int idSubhasta)
        {
            ResultPeticioParticipar result = new ResultPeticioParticipar();
            List<Subhasta> subhastes = this.GetSubhastes();
            Subhasta subhasta = subhastes.FirstOrDefault(s => s.IdSubhasta == idSubhasta);

            subhasta.Participants = this.GetLicitacions(subhasta.IdSubhasta);
            result.Subhasta = subhasta;

            //Es comproba si l'usuari ja participa en aquesta subhasta.
            Participant usuariPart =subhasta.Participants.FirstOrDefault(p => p.IdUsuari == usuari.IdUsuari);
            if (usuariPart == null)
            {
                //Si encara no hi participa i no arriba al màxim de participants s'afegeix com a participant de la 
                //subhasta.
                if (subhasta.Participants.Count >= subhasta.MaxParticipants)
                {
                    result.CodiError = -3;
                    result.MissatgeError = "La subhasta no accepta més participants";
                }
                else
                {
                    Participant participant = new Participant();
                    participant.IdUsuari = usuari.IdUsuari;
                    participant.IdSubhasta = subhasta.IdSubhasta;
                    participant.PreuMaxLicitacio = 0;
                    subhasta.Participants.Add(participant);
                    this.InsertParticipant(participant);
                }
            }

            return result;

        }

        /// <summary>
        /// Retorna la llista de participants incloent les licitacions que ha fet en la subhasta.
        /// </summary>
        /// <param name="idSubhasta"></param>
        /// <returns></returns>
        public List<Participant> GetLicitacions(int idSubhasta)
        {
            List<Participant> result = new List<Participant>();

            using (SqlConnection conn = this.GetConnection())
            {
                conn.Open();

                //S'obtenen els participants de la subhasta.
                StringBuilder sb = new StringBuilder();
                sb.AppendLine("SELECT p.IdParticipant, p.IdSubhasta, p.IdUsuari, p.PreuMaxLicitacio, u.Correu ");
                sb.AppendLine("FROM Participant p ");
                sb.AppendLine("INNER JOIN Usuari u ON p.IdUsuari = u.IdUsuari ");
                sb.AppendLine("WHERE IdSubhasta = @IdSubhasta");

                SqlCommand command = conn.CreateCommand();
                command.CommandText = sb.ToString();
                command.Parameters.AddWithValue("IdSubhasta", idSubhasta);
                IDataReader reader = command.ExecuteReader();
                while (reader.Read())
                {
                    Participant participant = new Participant();
                    participant.IdParticipant = reader.GetInt32(0);
                    participant.IdSubhasta = reader.GetInt32(1);
                    participant.IdUsuari = reader.GetInt32(2);
                    participant.PreuMaxLicitacio = reader.GetDecimal(3);
                    participant.Usuari = new Usuari {IdUsuari = participant.IdUsuari, Correu = reader.GetString(4) };

                    using (SqlConnection conn2 = this.GetConnection())
                    {
                        conn2.Open();

                        //S'obtenen les licitacions de cada participant.
                        sb = new StringBuilder();
                        sb.AppendLine("SELECT l.IdLicitacio, l.PreuLicitat, l.Data ");
                        sb.AppendLine("FROM Licitacio l");
                        sb.AppendLine("WHERE l.IdParticipant = @IdParticipant ");
                        SqlCommand command2 = conn2.CreateCommand();
                        command2.CommandText = sb.ToString();
                        command2.Parameters.AddWithValue("IdParticipant", participant.IdParticipant);
                        IDataReader readerLicitacio = command2.ExecuteReader();

                        while (readerLicitacio.Read())
                        {
                            Licitacio licitacio = new Licitacio();
                            licitacio.IdLicitacio = readerLicitacio.GetInt32(0);
                            licitacio.PreuLicitat = readerLicitacio.GetDecimal(1);
                            licitacio.Data = readerLicitacio.GetDateTime(2);
                            licitacio.IdParticipant = participant.IdParticipant;
                            licitacio.Usuari = participant.Usuari;
                            participant.Licitacions.Add(licitacio);
                        }

                        result.Add(participant);
                    }
                }
            }

            return result;
            
        }


        /// <summary>
        /// Inserta el registre en la taula Participant.
        /// </summary>
        /// <param name="usuari"></param>
        /// <param name="subhasta"></param>
        private void InsertParticipant(Participant participant)
        {
            using (SqlConnection conn = this.GetConnection())
            {
                conn.Open();
                StringBuilder sb = new StringBuilder();
                sb.AppendLine("INSERT INTO Participant (IdSubhasta, IdUsuari, PreuMaxLicitacio) ");
                sb.AppendLine("VALUES (@IdSubhasta, @IdUsuari, @PreuMaxLicitacio)");

                SqlCommand command = conn.CreateCommand();
                command.CommandText = sb.ToString();
                command.Parameters.AddWithValue("IdSubhasta", participant.IdSubhasta);
                command.Parameters.AddWithValue("IdUsuari", participant.IdUsuari);
                command.Parameters.AddWithValue("PreuMaxLicitacio", participant.PreuMaxLicitacio);
                command.ExecuteNonQuery();
            }
        }

        /// <summary>
        /// Afegeix la licitació a la llista de licitacions de la subhasta.
        /// </summary>
        /// <param name="participant"></param>
        /// <param name="preuLicitat"></param>
        /// <returns></returns>
        public List<Participant> Licitar(Participant participant, decimal preuLicitat)
        {
            List<Participant> result = this.GetLicitacions(participant.IdSubhasta);

            //Es busca l'històric de licitacions de l'usuari.
            Participant p = result.FirstOrDefault(f => f.IdUsuari == participant.IdUsuari);

            //S'afegeix la nova licitació.
            Licitacio licitacio = new Licitacio();
            licitacio.IdParticipant = p.IdParticipant;
            licitacio.PreuLicitat = preuLicitat;
            licitacio.Data = DateTime.Now;
            p.Licitacions.Add(licitacio);
            InsertLicitacio(licitacio);

            return result;

        }

        /// <summary>
        /// Inserta el registra a la taula Licitacio.
        /// </summary>
        /// <param name="licitacio"></param>
        private void InsertLicitacio(Licitacio licitacio)
        {
            using (SqlConnection conn = this.GetConnection())
            {
                conn.Open();
                StringBuilder sb = new StringBuilder();
                sb.AppendLine("INSERT INTO Licitacio (IdParticipant, PreuLicitat, Data) ");
                sb.AppendLine("VALUES (@IdParticipant, @PreuLicitat, @Data) ");

                SqlCommand command = conn.CreateCommand();
                command.CommandText = sb.ToString();
                command.Parameters.AddWithValue("IdParticipant", licitacio.IdParticipant);
                command.Parameters.AddWithValue("PreuLicitat", licitacio.PreuLicitat);
                command.Parameters.AddWithValue("Data", licitacio.Data);
                command.ExecuteNonQuery();
            }
        }


      

        /// <summary>
        /// Guarda l'estat i el preu actual de les subhastes indicades.
        /// </summary>
        /// <param name="subhastes"></param>
        public void GuardarEstatSubhastes(List<Subhasta> subhastes)
        {
            using (SqlConnection conn = this.GetConnection())
            {
                conn.Open();

                foreach (Subhasta subhasta in subhastes) {
                    StringBuilder sb = new StringBuilder();
                    sb.AppendLine("UPDATE Subhasta ");
                    sb.AppendLine("SET Estat=@Estat, ");
                    sb.AppendLine("    PreuActual=@PreuActual ");
                    sb.AppendLine("WHERE IdSubhasta=@IdSubhasta ");

                    SqlCommand command = conn.CreateCommand();
                    command.Parameters.AddWithValue("Estat", subhasta.Estat);
                    command.Parameters.AddWithValue("PreuActual", subhasta.PreuActual);
                    command.Parameters.AddWithValue("IdSubhasta", subhasta.IdSubhasta);
                    command.CommandText = sb.ToString();
                    command.ExecuteNonQuery();
                }
            }
        }
        
    }
}
