﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;

namespace SubhastesService.Entities
{
    
    public enum enumEstatSubhasta { Esperant=0, Iniciada=1, Finalitzada=2 }

    [DataContract]
    public class Subhasta
    {
        

        public Subhasta()
        {
            this.Participants = new HashSet<Participant>();
            
        }

        [DataMember]
        public int IdSubhasta { get; set; }
        [DataMember]
        public int IdProducte { get; set; }

        [DataMember]
        public Nullable<System.DateTime> DataInici { get; set; }
        [DataMember]
        public Nullable<System.DateTime> DataFi { get; set; }
        [DataMember]
        public Nullable<decimal> PreuSortida { get; set; }
        [DataMember]
        public Nullable<decimal> PreuActual { get; set; }
        [DataMember]
        public Nullable<int> MaxParticipants { get; set; }
        [DataMember]
        public Nullable<short> Estat { get; set; }

        [DataMember]
        public virtual ICollection<Participant> Participants { get; set; }

        [DataMember]
        public virtual Producte Producte { get; set; }


        public decimal GetPreuActual
        {
            get { return this.PreuActual.HasValue ? this.PreuActual.Value : 0; }
        }


        public enumEstatSubhasta GetEstat
        {
            get
            {
                if (this.Estat.HasValue)
                    return (enumEstatSubhasta)Enum.Parse(typeof(enumEstatSubhasta), this.Estat.Value.ToString());
                else
                    return enumEstatSubhasta.Esperant;
            }
        }


        /// <summary>
        /// Determina si la subhasta ja ha començat el temps que fa que s'ha iniciat.
        /// Si la subhasta encara no ha començat el temps que falta per a que s'inicii.
        /// </summary>
        public string TempsInici
        {
            get {
                if (!DataInici.HasValue)
                    return "-";
                
                TimeSpan timeSpan = DateTime.Now.Subtract(DataInici.Value);
                //return (int)timeSpan.TotalHours + timeSpan.ToString(@"\:mm\:ss");
                return timeSpan.ToString("dd") + " d " + timeSpan.ToString(@"hh\:mm\:ss");
                
            }
        }


        /// <summary>
        /// Determina el temps que falta per a que finalitzi la subhasta.
        /// </summary>
        public string TempsFi
        {
            get {
                    if (!DataFi.HasValue ||Estat!=(short)enumEstatSubhasta.Iniciada)
                        return "-";

                    TimeSpan timeSpan = DataFi.Value.Subtract(DateTime.Now);
                    //return (int)timeSpan.TotalHours + timeSpan.ToString(@"\:mm\:ss");
                    return timeSpan.ToString("dd") + " d " + timeSpan.ToString(@"hh\:mm\:ss");
                }
        }
       
    }
}
