﻿using SubhastesService.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;

namespace SubhastesService
{
    
    [ServiceContract]
    public interface IServeiSubhastes
    {
        /// <summary>
        /// Petició d'autenticació d'un usuari.
        /// Retorna un objecte ResultPeticioAutenticar.
        /// </summary>
        /// <param name="correu"></param>
        /// <param name="password"></param>
        /// <returns></returns>
        [OperationContract]
        ResultPeticioAutenticar Autenticar(string correu, string password);

        /// <summary>
        /// Retorna una llista de subhastes actives. 
        /// </summary>
        /// <returns></returns>
        [OperationContract]
        List<Subhasta> GetSubhastes();

        /// <summary>
        /// Retorna la subhasta indicada.
        /// </summary>
        /// <returns></returns>
        [OperationContract]
        Subhasta GetSubhasta(int idSubhasta);

        /// <summary>
        /// Registra l'usuari indicat.
        /// Retorna un objecte ResultPetició.
        /// </summary>
        /// <param name="usuari"></param>
        /// <returns></returns>
        [OperationContract]
        ResultPeticioAutenticar RegistrarUsuari(Usuari usuari);

        /// <summary>
        /// Petició de participació en una subhasta. (Join)
        /// Retorna un objecte ResultPeticioParticipar. 
        /// </summary>
        /// <param name="usuari"></param>
        /// <param name="idSubhasta"></param>
        /// <returns></returns>
        [OperationContract]
        ResultPeticioParticipar Participar(Usuari usuari, int idSubhasta);

        /// <summary>
        /// Petició de licitació. (Bid)
        /// </summary>
        /// <param name="licitacio"></param>
        [OperationContract]
        List<Participant> Licitar(Participant participant, decimal preuLicitat);

        /// <summary>
        /// Petició de les últimes licitacions.
        /// </summary>
        /// <returns></returns>
        [OperationContract]
        List<Participant> GetLicitacions(int idSubhasta);
     }

   
}
