﻿using SubhastesService.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;
using System.Timers;

namespace SubhastesService
{
    
    public class ServeiSubhastes : IServeiSubhastes
    {
        
        public ServeiSubhastes()
        {
            this.Control();

            System.Console.Out.WriteLine("Servei iniciat");
            
        }


        /// <summary>
        /// Petició d'autenticació d'un usuari.
        /// Retorna un objecte ResultPeticioAutenticar.
        /// </summary>
        /// <param name="correu"></param>
        /// <param name="password"></param>
        /// <returns></returns>
        public ResultPeticioAutenticar Autenticar(string correu, string password)
        {
            return DataService.Instance.Autenticacio.Autenticar(correu, password);
        }


        /// <summary>
        /// Retorna una llista de subhastes actives. 
        /// </summary>
        /// <returns></returns>
        public List<Subhasta> GetSubhastes()
        {
            return DataService.Instance.Subhasta.GetSubhastes();
        }


        /// <summary>
        /// Retorna la subhasta indicada.
        /// </summary>
        /// <returns></returns>
        public Subhasta GetSubhasta(int idSubhasta)
        {
            return DataService.Instance.Subhasta.GetSubhasta(idSubhasta);
        }


        /// <summary>
        /// Registra l'usuari indicat.
        /// Retorna un objecte ResultPetició.
        /// </summary>
        /// <param name="usuari"></param>
        /// <returns></returns>
        public ResultPeticioAutenticar RegistrarUsuari(Usuari usuari)
        {
            return DataService.Instance.Autenticacio.RegistrarUsuari(usuari);
        }


        /// <summary>
        /// Petició de participació en una subhasta. (Join)
        /// Retorna un objecte ResultPeticioParticipar. 
        /// </summary>
        /// <param name="usuari"></param>
        /// <param name="idSubhasta"></param>
        /// <returns></returns>
        public ResultPeticioParticipar Participar(Usuari usuari, int idSubhasta)
        {
            return DataService.Instance.Subhasta.Participar(usuari, idSubhasta);
        }


        /// <summary>
        /// Petició de licitació. (Bid)
        /// </summary>
        /// <param name="licitacio"></param>
        public List<Participant> Licitar(Participant participant, decimal preuLicitat)
        {
            return DataService.Instance.Subhasta.Licitar(participant, preuLicitat);
        }


        /// <summary>
        /// Petició de les últimes licitacions.
        /// </summary>
        /// <returns></returns>
        public List<Participant> GetLicitacions(int idSubhasta)
        {
            return DataService.Instance.Subhasta.GetLicitacions(idSubhasta);
        }


        /// <summary>
        /// Gestiona l'estat de les subhastes y el preu actual en funció de les licitacions.
        /// </summary>
        private void Control()
        {
            List<Subhasta> subhastesPerIniciar = this.GetSubhastes();
            List<Subhasta> subhastesEstatCambiat = new List<Subhasta>();

            foreach (Subhasta subhasta in subhastesPerIniciar)
            {
                if (!subhasta.DataInici.HasValue || !subhasta.DataInici.HasValue)
                    continue;

                //Es gestiona l'estat de la subhasta.
                DateTime ahora = DateTime.Now;

                switch (subhasta.GetEstat)
                {
                    case enumEstatSubhasta.Esperant:
                        //Si la data d'ara es superior o igual a la data d'inici i es inferior a la data de fi 
                        //s'inicia la subhasta.
                        if (ahora.Subtract(subhasta.DataInici.Value).TotalSeconds > 0 &&
                            ahora.Subtract(subhasta.DataFi.Value).TotalSeconds <= 0)
                        {
                            subhasta.Estat = (short)enumEstatSubhasta.Iniciada;
                            subhastesEstatCambiat.Add(subhasta);
                        }
                        break;
                    case enumEstatSubhasta.Iniciada:
                        //Si la data d'ara es superior o igual a la data de fi es finalitza la subhasta.
                        if (ahora.Subtract(subhasta.DataFi.Value).TotalSeconds >= 0)
                        {
                            subhasta.Estat = (short) enumEstatSubhasta.Finalitzada;
                            subhastesEstatCambiat.Add(subhasta);
                        }
                        break;
                    case enumEstatSubhasta.Finalitzada:
                        break;
                 
                }

                //Es calcula el preu actual en funció del preu de sortida y les licitacions fetes fins ara.
                List<Participant> participants = DataService.Instance.Subhasta.GetLicitacions(subhasta.IdSubhasta);
                decimal preuActual = subhasta.PreuSortida.Value;
                foreach (Participant participant in participants)
                {
                    preuActual += participant.Licitacions.Sum(l => l.PreuLicitat.Value);
                }

                //Si ha cambiat el preu de la subhasta s'afegeix a la llista de subhastes per guardar.
                if (preuActual != subhasta.PreuActual &&
                    !subhastesEstatCambiat.Contains(subhasta))
                {
                    subhasta.PreuActual = preuActual;
                    subhastesEstatCambiat.Add(subhasta);
                }
            }

            if (subhastesEstatCambiat.Count>0)
                DataService.Instance.Subhasta.GuardarEstatSubhastes(subhastesEstatCambiat);
        }
    }
}
